﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using STM.Common.DataAccess;
using STM.Common.DataAccess.Entities;
using System.Data.Entity;
using STM.Common.Services.Internal.Interfaces;

namespace STM.Common.Services.Internal
{
    public class LogEventService : InternalServiceBase<LogEvent, LogDbContext>, ILogEventService
    {
        private EventDataType _messageType { get; set; }

        public LogEventService(LogDbContext dbContext) : base(dbContext)
        {
        }

        public void Init(EventDataType messageType)
        {
            _messageType = messageType;
        }

        public void LogError(EventNumber eventNumber, EventType eventType,
            List<KeyValuePair<string, string>> parameters, string eventData)
        {

            var logEvent = new LogEvent
            {
                EventTime = SetDate(),
                ServiceInstanceId = Common.InstanceContext.ServiceId,
                EventNumber = eventNumber,
                EventType = eventType,
                ExternalOrgId = Common.InstanceContext.CallerOrgId,
                ExternalEntityId = Common.InstanceContext.CallerServiceId,
                EventParameters = ConvertToString(parameters),
                EventDataType = _messageType,
                EventData = eventData
            };

            base.Insert(logEvent);

        }

        private string ConvertToString(List<KeyValuePair<string, string>> parameters)
        {
            string result = string.Empty;

            if (parameters != null && parameters.Count() > 0)
            {
                foreach (var parameter in parameters)
                {
                    result += parameter.Key.ToString() + " = " + parameter.Value + ", ";
                }
                result = result.TrimEnd();
                result = result.Substring(0, result.Length - 1);
            }

            return result;
        }

        private DateTime SetDate()
        {
            return DateTime.UtcNow.ToUniversalTime();
        }

        public void LogSuccess(EventNumber eventNumber,
            List<KeyValuePair<string, string>> parameters, string eventData)
        {

            var logEvent = new LogEvent
            {
                EventTime = SetDate(),
                ServiceInstanceId = Common.InstanceContext.ServiceId,
                EventNumber = eventNumber,
                EventType = EventType.Successful,
                ExternalOrgId = Common.InstanceContext.CallerOrgId,
                ExternalEntityId = Common.InstanceContext.CallerServiceId,
                EventParameters = ConvertToString(parameters),
                EventDataType = _messageType,
                EventData = eventData
            };

            base.Insert(logEvent);
        }

        public void LogInfo(EventNumber eventNumber,
            List<KeyValuePair<string, string>> parameters, string eventData)
        {
            var logEvent = new LogEvent
            {
                EventTime = SetDate(),
                ServiceInstanceId = Common.InstanceContext.ServiceId,
                EventNumber = eventNumber,
                EventType = EventType.Info,
                ExternalOrgId = Common.InstanceContext.CallerOrgId,
                ExternalEntityId = Common.InstanceContext.CallerServiceId,
                EventParameters = ConvertToString(parameters),
                EventDataType = _messageType,
                EventData = eventData
            };

            base.Insert(logEvent);
        }

        //public EventCategory StatusCodeToEventCategory(HttpStatusCode statusCode)
        //{
        //    switch(statusCode)
        //    {
        //        case HttpStatusCode.OK:
        //            return EventCategory.Success;
        //        case HttpStatusCode.BadRequest:
        //            return EventCategory.RequestError;
        //        case HttpStatusCode.Forbidden:
        //            return EventCategory.AuthorizationError;
        //        case HttpStatusCode.InternalServerError:
        //            return EventCategory.InternalError;
        //        case HttpStatusCode.NotFound:
        //            return EventCategory.RequestError;
        //        case HttpStatusCode.NotImplemented:
        //            return EventCategory.InternalError;
        //        case HttpStatusCode.ProxyAuthenticationRequired:
        //            return EventCategory.AuthenticationError;
        //        case HttpStatusCode.RequestEntityTooLarge:
        //            return EventCategory.InternalError;
        //        case HttpStatusCode.RequestTimeout:
        //            return EventCategory.InternalError;
        //        case HttpStatusCode.RequestUriTooLong:
        //            return EventCategory.RequestError;
        //        case HttpStatusCode.ServiceUnavailable:
        //            return EventCategory.CommunicationError;
        //        case HttpStatusCode.Unauthorized:
        //            return EventCategory.AuthorizationError;
        //        case HttpStatusCode.SeeOther:
        //            return EventCategory.InternalError;
        //        default:
        //            return EventCategory.UnknownError;
        //    }

        //}
    }
}
