﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace STM.Common
{
    public class WebRequestHelper
    {
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        public static WebResponse Get(string url, WebHeaderCollection headers = null, bool UseCertificate = false)
        {
            var result = ExecuteWebRequest(url, null, "GET", headers, UseCertificate);
            return result;
        }

        public static WebResponse Post(string url, string body, WebHeaderCollection headers = null, bool UseCertificate = false)
        {
            var result = ExecuteWebRequest(url, body, "POST", headers, UseCertificate);
            return result;
        }

        public static WebResponse Put(string url, string body, WebHeaderCollection headers = null, bool UseCertificate = false)
        {
            var result = ExecuteWebRequest(url, body, "PUT", headers, UseCertificate);
            return result;
        }

        public static WebResponse Delete(string url, string body, WebHeaderCollection headers = null, bool UseCertificate = false)
        {
            var result = ExecuteWebRequest(url, body, "DELETE", headers, UseCertificate);
            return result;
        }

        private static WebResponse ExecuteWebRequest(string url, string body, string method, WebHeaderCollection headers = null, bool UseCertificate = false)
        {
            log.Debug(string.Format("ExecuteWebRequest url: {0}, body: {1}, method: {2}, UseCertificate: {3}", url, body, method, UseCertificate.ToString()));

            HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;
            request.Method = method;

            if (headers != null)
            {
                foreach (string key in headers.AllKeys)
                {
                    if (key.ToLower() == "content-type")
                    {
                        request.ContentType = headers[key];
                        headers.Remove(key);
                    }
                }

                if (request.Headers == null)
                    request.Headers = headers;
                else
                {
                    foreach (string key in headers.AllKeys)
                        request.Headers.Add(key, headers[key]);
                }
            }

            if (UseCertificate)
            {
                if (InstanceContext.ClientCertificate == null)
                    throw new Exception("Client certificate has not been set");

                request.ClientCertificates.Add(InstanceContext.ClientCertificate);
            }

            if (method == "POST" || method == "DELETE")
            {
                var byteArray = System.Text.Encoding.UTF8.GetBytes(body ?? string.Empty);
                request.ContentLength = byteArray.Length;
                using (var dataStream = request.GetRequestStream())
                {
                    dataStream.Write(byteArray, 0, byteArray.Length);
                }
            }

            var result = new WebResponse();
            request.PreAuthenticate = true;
            try
            {
                var response = (HttpWebResponse)request.GetResponse();

                result.HttpStatusCode = response.StatusCode;

                using (var sr = new StreamReader(response.GetResponseStream()))
                {
                    result.Body = sr.ReadToEnd();
                }


                log.Debug("Response: " + response.StatusCode + ", " + result.Body);
            }
            catch (WebException wex)
            {
                log.Error(wex);
                if (wex.Response != null)
                {
                    using (var sr = new StreamReader(wex.Response.GetResponseStream()))
                    {
                        result.Body = sr.ReadToEnd();
                    }
                    result.HttpStatusCode = ((HttpWebResponse)wex.Response).StatusCode;
                }
                else
                {
                    result.HttpStatusCode = HttpStatusCode.InternalServerError;
                }

                result.ErrorMessage = wex.Message;
            }

            return result;
        }

        public class WebResponse
        {
            public HttpStatusCode HttpStatusCode { get; set; }
            public string ErrorMessage { get; set; }
            public string Body { get; set; }
        }
    }
}
