﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

namespace STM.Common
{
    public static class XMLParser
    {
        public static string GetTextMessageId(string textMessageXML)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(textMessageXML);
            XmlNode root = doc.DocumentElement;

            XmlNamespaceManager nsmgr = new XmlNamespaceManager(doc.NameTable);
            nsmgr.AddNamespace("mstns", "http://tempuri.org/TextFormat.xsd");

            var node = root.SelectSingleNode("//mstns:textMessageId", nsmgr);
            if (node != null)
                return node.InnerText;
            else
                return string.Empty;
        }

        public static string GetAreaMessageID(string dataSet)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(dataSet);
            XmlNode root = doc.DocumentElement;

            var node = root.SelectSingleNode("//referenceUVID");

            if (node != null)
                return node.InnerText;
            else
                return string.Empty;
        }

        public static string GetRouteStatus(string dataSet)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(dataSet);
            XmlNode root = doc.DocumentElement;

            XmlNamespaceManager nsmgr = new XmlNamespaceManager(doc.NameTable);
            nsmgr.AddNamespace("rt", "http://www.cirm.org/RTZ/1/0");

            var node = root.SelectSingleNode("//rt:routeInfo", nsmgr);
            
            if (node != null)
                return node.Attributes["routeStatus"] != null ? node.Attributes["routeStatus"].Value : null; 
            else
                return string.Empty;
        }

        public static string GetRouteInfo(string dataSet)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(dataSet);
            XmlNode root = doc.DocumentElement;

            XmlNamespaceManager nsmgr = new XmlNamespaceManager(doc.NameTable);
            nsmgr.AddNamespace("rt", "http://www.cirm.org/RTZ/1/0");

            var node = root.SelectSingleNode("//rt:routeInfo", nsmgr);

            if (node != null)
                return node.OuterXml;
            else
                return string.Empty;
        }

        public static string GetWayPoints(string dataSet)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(dataSet);
            XmlNode root = doc.DocumentElement;

            XmlNamespaceManager nsmgr = new XmlNamespaceManager(doc.NameTable);
            nsmgr.AddNamespace("rt", "http://www.cirm.org/RTZ/1/0");

            var node = root.SelectSingleNode("//rt:waypoints", nsmgr);

            if (node != null)
                return node.OuterXml;
            else
                return string.Empty;
        }

        public static string GetSchedules(string dataSet)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(dataSet);
            XmlNode root = doc.DocumentElement;

            XmlNamespaceManager nsmgr = new XmlNamespaceManager(doc.NameTable);
            nsmgr.AddNamespace("rt", "http://www.cirm.org/RTZ/1/0");

            var node = root.SelectSingleNode("//rt:schedules", nsmgr);

            if (node != null)
                return node.OuterXml;
            else
                return string.Empty;
        }

        public static string GetVesselVoyage(string dataSet)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(GetRouteInfo(dataSet));
            XmlNode root = doc.DocumentElement;

            if (root != null)
                return root.Attributes["vesselVoyage"] != null ? root.Attributes["vesselVoyage"].Value : null;
            else
                return string.Empty;
        }

        public static DateTime? GetValidityPeriodStart(string dataSet)
        {
            string result;
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(GetRouteInfo(dataSet));
            XmlNode root = doc.DocumentElement;


            if (root != null)
            {
                result = root.Attributes["validityPeriodStart"] != null ? root.Attributes["validityPeriodStart"].Value : null;
                return Convert.ToDateTime(result);
            }

            else
                return null;
        }

        public static string GetPcmMessageId(string pcmMessageObject)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(pcmMessageObject);
            XmlNode root = doc.DocumentElement;

            if (root != null)
            {
                return root.Attributes["messageId"] != null ? root.Attributes["messageId"].Value : null;
            }
            else
                return null;
        }

        public static DateTime? GetValidityPeriodStop(string dataSet)
        {
            string result;
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(GetRouteInfo(dataSet));
            XmlNode root = doc.DocumentElement;


            if (root != null)
            {
                result = root.Attributes["validityPeriodStop"] != null ? root.Attributes["validityPeriodStop"].Value : null;
                return Convert.ToDateTime(result);
            }
            else
                return null;
        }
    }
}
