﻿using Microsoft.Win32;
using STM.StmModule.Simulator.Contract;
using STM.StmModule.Simulator.Infrastructure;
using STM.StmModule.Simulator.Services;

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Xml;

namespace STM.StmModule.Simulator.ViewModels
{
    public class GetMessagesViewModel : ViewModelBase
    {
        public GetMessagesViewModel()
        {
        }

        private string _id;
        public string Id
        {
            get
            {
                return _id;
            }
            set
            {
                if (_id == value)
                    return;

                _id = value;

                OnPropertyChanged(() => Id);
            }
        }

        private int _limitQuery = 10;
        public int LimitQuery
        {
            get
            {
                return _limitQuery;
            }
            set
            {
                if (_limitQuery == value)
                    return;

                _limitQuery = value;

                OnPropertyChanged(() => LimitQuery);
            }
        }

        private int _remainingMessages;
        public int RemainingMessages
        {
            get
            {
                return _remainingMessages;
            }
            set
            {
                if (_remainingMessages == value)
                    return;

                _remainingMessages = value;

                OnPropertyChanged(() => RemainingMessages);
            }
        }

        private Message _selectedMessage;
        public Message SelectedMessage
        {
            get
            {
                return _selectedMessage;
            }
            set
            {
                if (_selectedMessage == value)
                    return;

                _selectedMessage = value;

                OnPropertyChanged(() => SelectedMessage);
            }
        }

        private ObservableCollection<Message> _messages = new ObservableCollection<Message>();
        public ObservableCollection<Message> Messages
        {
            get
            {
                return _messages;
            }
            set
            {
                if (_messages == value)
                    return;

                _messages = value;

                OnPropertyChanged(() => Messages);
            }
        }


        private ICommand _getMessagesCommand;
        public ICommand GetMessagesCommand
        {
            get
            {
                return _getMessagesCommand ??
                    (_getMessagesCommand = new DelegateCommand(ExecuteGetMessagesCommand, CanExecuteGetMessagesCommand));
            }
        }

        public bool CanExecuteGetMessagesCommand(object parameter)
        {
            return true;
        }

        public void ExecuteGetMessagesCommand(object parameter)
        {
            var visService = new VisService();

            var result = visService.GetMessages(Id, LimitQuery);

            RemainingMessages = result.RemainingNumberOfMessages ?? 0;

            if (result.Messages != null)
            {
                foreach (var message in result.Messages)
                {
                    Messages.Add(message);
                }
            }
        }

        private ICommand _getPCMMessagesCommand;

        public ICommand GetPCMMessagesCommand
        {
            get
            {
                return _getPCMMessagesCommand ??
                    (_getPCMMessagesCommand = new DelegateCommand(ExecuteGetPCMMessagesCommand, CanExecuteGetPCMMessagesCommand));
            }
        }

        private bool CanExecuteGetPCMMessagesCommand(object obj)
        {
            return true;
        }

        private void ExecuteGetPCMMessagesCommand(object obj)
        {
            var service = new SpisService();

            var result = service.GetMessages(Id, LimitQuery);

            RemainingMessages = result.RemainingNumberOfMessages ?? 0;

            if (result.Messages != null)
            {
                foreach (var message in result.Messages)
                {
                    Messages.Add(message);
                }
            }
        }
    }
}
